@@ -64,7 +64,7 @@ int debug_dump_active = 0;
 void
 debug_dump (int crash)
 {
-    /* prevent reentrance */
+    /* prevent reentrancy */
     if (debug_dump_active)
         exit (EXIT_FAILURE);
 
@@ -156,7 +156,7 @@ debug_sigsegv ()
                               weechat_home);
     string_iconv_fprintf (stderr, "***\n");
     string_iconv_fprintf (stderr, "*** Please help WeeChat developers to fix this bug:\n");
-    string_iconv_fprintf (stderr, "***   1. If you have a core file, please run:  gdb weechat-curses core\n");
+    string_iconv_fprintf (stderr, "***   1. If you have a core file, please run:  gdb /path/to/weechat core\n");
     string_iconv_fprintf (stderr, "***      then issue \"bt\" command and send result to developers\n");
     string_iconv_fprintf (stderr, "***      To enable core files with bash shell: ulimit -c 10000\n");
     string_iconv_fprintf (stderr, "***   2. Otherwise send backtrace (below) and weechat.log\n");
@@ -369,10 +369,8 @@ debug_hdata_map_cb (void *data, struct t_hashtable *hashtable,
     gui_chat_printf (NULL,
                      "  hdata 0x%lx: \"%s\", %d vars, %d lists:",
                      ptr_hdata, (const char *)key,
-                     hashtable_get_integer (ptr_hdata->hash_var,
-                                            "items_count"),
-                     hashtable_get_integer (ptr_hdata->hash_list,
-                                            "items_count"));
+                     ptr_hdata->hash_var->items_count,
+                     ptr_hdata->hash_list->items_count);
 
     /* display lists */
     hashtable_map (ptr_hdata->hash_list,
@@ -412,7 +410,7 @@ debug_hdata ()
 {
     int count;
 
-    count = hashtable_get_integer (weechat_hdata, "items_count");
+    count = weechat_hdata->items_count;
 
     gui_chat_printf (NULL, "");
     gui_chat_printf (NULL, "%d hdata in memory", count);
@@ -473,7 +471,7 @@ debug_infolists ()
     gui_chat_printf (NULL, "");
     gui_chat_printf (NULL, "%d infolists in memory (%s)", count,
                      (count == 0) ?
-                     "this is ok!" :
+                     "this is OK!" :
                      "WARNING: this is probably a memory leak in WeeChat or "
                      "plugins/scripts!");
 
@@ -539,6 +537,22 @@ debug_infolists ()
     }
 }
 
+/*
+ * Displays WeeChat directories.
+ */
+
+void
+debug_directories ()
+{
+    gui_chat_printf (NULL, "");
+    gui_chat_printf (NULL, _("Directories:"));
+    gui_chat_printf (NULL, "  home  : %s (%s: %s)",
+                     weechat_home, _("default"), WEECHAT_HOME);
+    gui_chat_printf (NULL, "  lib   : %s", WEECHAT_LIBDIR);
+    gui_chat_printf (NULL, "  share : %s", WEECHAT_SHAREDIR);
+    gui_chat_printf (NULL, "  locale: %s", LOCALEDIR);
+}
+
 /*
  * Hooks signals for debug.
  */
