@@ -27,8 +27,8 @@ struct t_hashtable;
 
 enum t_eval_logical_op
 {
-    EVAL_LOGICAL_OP_AND = 0,
-    EVAL_LOGICAL_OP_OR,
+    EVAL_LOGICAL_OP_OR = 0,
+    EVAL_LOGICAL_OP_AND,
     /* number of comparison strings */
     EVAL_NUM_LOGICAL_OPS,
 };
@@ -50,6 +50,8 @@ enum t_eval_comparison
 extern int eval_is_true (const char *value);
 extern char *eval_expression (const char *expr,
                               struct t_hashtable *pointers,
-                              struct t_hashtable *extra_vars);
+                              struct t_hashtable *extra_vars,
+                              struct t_hashtable *options);
+extern void eval_end ();
 
 #endif /* __WEECHAT_EVAL_H */
