@@ -31,6 +31,7 @@
 #include <wctype.h>
 #include <regex.h>
 #include <wchar.h>
+#include <stdint.h>
 
 #ifdef HAVE_ICONV
 #include <iconv.h>
@@ -47,11 +48,17 @@
 #include "weechat.h"
 #include "wee-string.h"
 #include "wee-config.h"
+#include "wee-hashtable.h"
 #include "wee-utf8.h"
 #include "../gui/gui-color.h"
 #include "../plugins/plugin.h"
 
 
+typedef uint32_t string_shared_count_t;
+
+struct t_hashtable *string_hashtable_shared = NULL;
+
+
 /*
  * Defines a "strndup" function for systems where this function does not exist
  * (FreeBSD and maybe others).
@@ -845,6 +852,10 @@ string_regex_flags (const char *regex, int default_flags, int *flags)
 /*
  * Compiles a regex using optional flags at beginning of string (for format of
  * flags in regex, see string_regex_flags()).
+ *
+ * Returns:
+ *   0: successful compilation
+ *   other value: compilation failed
  */
 
 int
@@ -1048,6 +1059,9 @@ string_has_highlight_regex (const char *string, const char *regex)
 /*
  * Splits a string according to separators.
  *
+ * This function must not be called directly (call string_split or
+ * string_split_shared instead).
+ *
  * Examples:
  *   string_split ("abc de  fghi", " ", 0, 0, NULL)
  *     ==> array[0] = "abc"
@@ -1062,12 +1076,13 @@ string_has_highlight_regex (const char *string, const char *regex)
  */
 
 char **
-string_split (const char *string, const char *separators, int keep_eol,
-              int num_items_max, int *num_items)
+string_split_internal (const char *string, const char *separators, int keep_eol,
+                       int num_items_max, int *num_items, int shared)
 {
     int i, j, n_items;
     char *string2, **array;
     char *ptr, *ptr1, *ptr2;
+    const char *str_shared;
 
     if (num_items != NULL)
         *num_items = 0;
@@ -1137,13 +1152,18 @@ string_split (const char *string, const char *separators, int keep_eol,
             {
                 if (keep_eol)
                 {
-                    array[i] = strdup (ptr1);
+                    array[i] = (shared) ? (char *)string_shared_get (ptr1) : strdup (ptr1);
                     if (!array[i])
                     {
                         for (j = 0; j < n_items; j++)
                         {
                             if (array[j])
-                                free (array[j]);
+                            {
+                                if (shared)
+                                    string_shared_free (array[j]);
+                                else
+                                    free (array[j]);
+                            }
                         }
                         free (array);
                         free (string2);
@@ -1158,7 +1178,12 @@ string_split (const char *string, const char *separators, int keep_eol,
                         for (j = 0; j < n_items; j++)
                         {
                             if (array[j])
-                                free (array[j]);
+                            {
+                                if (shared)
+                                    string_shared_free (array[j]);
+                                else
+                                    free (array[j]);
+                            }
                         }
                         free (array);
                         free (string2);
@@ -1166,6 +1191,23 @@ string_split (const char *string, const char *separators, int keep_eol,
                     }
                     strncpy (array[i], ptr1, ptr2 - ptr1);
                     array[i][ptr2 - ptr1] = '\0';
+                    if (shared)
+                    {
+                        str_shared = string_shared_get (array[i]);
+                        if (!str_shared)
+                        {
+                            for (j = 0; j < n_items; j++)
+                            {
+                                if (array[j])
+                                    string_shared_free (array[j]);
+                            }
+                            free (array);
+                            free (string2);
+                            return NULL;
+                        }
+                        free (array[i]);
+                        array[i] = (char *)str_shared;
+                    }
                 }
                 ptr1 = ++ptr2;
             }
@@ -1185,6 +1227,35 @@ string_split (const char *string, const char *separators, int keep_eol,
     return array;
 }
 
+/*
+ * Splits a string according to separators.
+ *
+ * For full description, see function string_split_internal.
+ */
+
+char **
+string_split (const char *string, const char *separators, int keep_eol,
+              int num_items_max, int *num_items)
+{
+    return string_split_internal (string, separators, keep_eol,
+                                  num_items_max, num_items, 0);
+}
+
+/*
+ * Splits a string according to separators, and use shared strings for the
+ * strings in the array returned.
+ *
+ * For full description, see function string_split_internal.
+ */
+
+char **
+string_split_shared (const char *string, const char *separators, int keep_eol,
+                     int num_items_max, int *num_items)
+{
+    return string_split_internal (string, separators, keep_eol,
+                                  num_items_max, num_items, 1);
+}
+
 /*
  * Splits a string like the shell does for a command with arguments.
  *
@@ -1394,6 +1465,23 @@ string_free_split (char **split_string)
     }
 }
 
+/*
+ * Frees a split string (using shared strings).
+ */
+
+void
+string_free_split_shared (char **split_string)
+{
+    int i;
+
+    if (split_string)
+    {
+        for (i = 0; split_string[i]; i++)
+            string_shared_free (split_string[i]);
+        free (split_string);
+    }
+}
+
 /*
  * Builds a string with a split string.
  *
@@ -1656,7 +1744,7 @@ string_iconv (int from_utf8, const char *from_code, const char *to_code,
 /*
  * Converts a string to WeeChat internal storage charset (UTF-8).
  *
- * Note: result has to be freed after use.
+ * Note: result must be freed after use.
  */
 
 char *
@@ -1696,7 +1784,7 @@ string_iconv_to_internal (const char *charset, const char *string)
 /*
  * Converts internal string to terminal charset, for display.
  *
- * Note: result has to be freed after use.
+ * Note: result must be freed after use.
  */
 
 char *
@@ -1762,7 +1850,7 @@ string_iconv_fprintf (FILE *file, const char *data, ...)
 /*
  * Formats a string with size and unit name (bytes, KB, MB, GB).
  *
- * Note: result has to be freed after use.
+ * Note: result must be freed after use.
  */
 
 char *
@@ -1796,6 +1884,78 @@ string_format_size (unsigned long long size)
     return strdup (str_size);
 }
 
+/*
+ * Encodes a string in base16 (hexadecimal).
+ *
+ * Argument "length" is number of bytes in "from" to convert (commonly
+ * strlen(from)).
+ */
+
+void
+string_encode_base16 (const char *from, int length, char *to)
+{
+    int i;
+    const char *hexa = "0123456789ABCDEF";
+    char *ptr_to;
+
+    ptr_to = to;
+    ptr_to[0] = '\0';
+    for (i = 0; i < length; i++)
+    {
+        ptr_to[0] = hexa[((unsigned char)from[i]) / 16];
+        ptr_to[1] = hexa[((unsigned char)from[i]) % 16];
+        ptr_to += 2;
+    }
+    ptr_to[0] = '\0';
+}
+
+/*
+ * Decodes a base16 string (hexadecimal).
+ *
+ * Returns length of string in "*to" (it does not count final \0).
+ */
+
+int
+string_decode_base16 (const char *from, char *to)
+{
+    int length, to_length, i, pos;
+    unsigned char *ptr_to, value;
+
+    length = strlen (from) / 2;
+
+    ptr_to = (unsigned char *)to;
+    ptr_to[0] = '\0';
+    to_length = 0;
+
+    for (i = 0; i < length; i++)
+    {
+        pos = i * 2;
+        value = 0;
+        /* 4 bits on the left */
+        if ((from[pos] >= '0') && (from[pos] <= '9'))
+            value |= (from[pos] - '0') << 4;
+        else if ((from[pos] >= 'a') && (from[pos] <= 'f'))
+            value |= (from[pos] - 'a' + 10) << 4;
+        else if ((from[pos] >= 'A') && (from[pos] <= 'F'))
+            value |= (from[pos] - 'A' + 10) << 4;
+        /* 4 bits on the right */
+        pos++;
+        if ((from[pos] >= '0') && (from[pos] <= '9'))
+            value |= from[pos] - '0';
+        else if ((from[pos] >= 'a') && (from[pos] <= 'f'))
+            value |= from[pos] - 'a' + 10;
+        else if ((from[pos] >= 'A') && (from[pos] <= 'F'))
+            value |= from[pos] - 'A' + 10;
+
+        ptr_to[0] = value;
+        ptr_to++;
+        to_length++;
+    }
+    ptr_to[0] = '\0';
+
+    return to_length;
+}
+
 /*
  * Converts 3 bytes of 8 bits in 4 bytes of 6 bits.
  */
@@ -2027,23 +2187,31 @@ string_input_for_buffer (const char *string)
  * must be newly allocated because it will be freed in this function).
  *
  * Argument "errors" is set with number of keys not found by callback.
+ *
+ * Note: result must be freed after use.
  */
 
 char *
 string_replace_with_callback (const char *string,
+                              const char *prefix,
+                              const char *suffix,
                               char *(*callback)(void *data, const char *text),
                               void *callback_data,
                               int *errors)
 {
-    int length, length_value, index_string, index_result;
+    int length_prefix, length_suffix, length, length_value, index_string;
+    int index_result;
     char *result, *result2, *key, *value;
     const char *pos_end_name;
 
     *errors = 0;
 
-    if (!string)
+    if (!string || !prefix || !prefix[0] || !suffix || !suffix[0])
         return NULL;
 
+    length_prefix = strlen (prefix);
+    length_suffix = strlen (suffix);
+
     length = strlen (string) + 1;
     result = malloc (length);
     if (result)
@@ -2053,19 +2221,18 @@ string_replace_with_callback (const char *string,
         while (string[index_string])
         {
             if ((string[index_string] == '\\')
-                && (string[index_string + 1] == '$'))
+                && (string[index_string + 1] == prefix[0]))
             {
                 index_string++;
                 result[index_result++] = string[index_string++];
             }
-            else if ((string[index_string] == '$')
-                     && (string[index_string + 1] == '{'))
+            else if (strncmp (string + index_string, prefix, length_prefix) == 0)
             {
-                pos_end_name = strchr (string + index_string + 2, '}');
+                pos_end_name = strstr (string + index_string + length_prefix, suffix);
                 if (pos_end_name)
                 {
-                    key = string_strndup (string + index_string + 2,
-                                          pos_end_name - (string + index_string + 2));
+                    key = string_strndup (string + index_string + length_prefix,
+                                          pos_end_name - (string + index_string + length_prefix));
                     if (key)
                     {
                         value = (*callback) (callback_data, key);
@@ -2086,7 +2253,7 @@ string_replace_with_callback (const char *string,
                             strcpy (result + index_result, value);
                             index_result += length_value;
                             index_string += pos_end_name - string -
-                                index_string + 1;
+                                index_string + length_suffix;
                             free (value);
                         }
                         else
@@ -2094,7 +2261,6 @@ string_replace_with_callback (const char *string,
                             result[index_result++] = string[index_string++];
                             (*errors)++;
                         }
-
                         free (key);
                     }
                     else
@@ -2111,3 +2277,163 @@ string_replace_with_callback (const char *string,
 
     return result;
 }
+
+/*
+ * Hashes a shared string.
+ * The string starts after the reference count, which is skipped.
+ *
+ * Returns the hash of the shared string (variant of djb2).
+ */
+
+unsigned long
+string_shared_hash_key (struct t_hashtable *hashtable,
+                        const void *key)
+{
+    /* make C compiler happy */
+    (void) hashtable;
+
+    return hashtable_hash_key_djb2 (((const char *)key) + sizeof (string_shared_count_t));
+}
+
+/*
+ * Compares two shared strings.
+ * Each string starts after the reference count, which is skipped.
+ *
+ * Returns:
+ *   < 0: key1 < key2
+ *     0: key1 == key2
+ *   > 0: key1 > key2
+ */
+
+int
+string_shared_keycmp (struct t_hashtable *hashtable,
+                      const void *key1, const void *key2)
+{
+    /* make C compiler happy */
+    (void) hashtable;
+
+    return strcmp (((const char *)key1) + sizeof (string_shared_count_t),
+                   ((const char *)key2) + sizeof (string_shared_count_t));
+}
+
+/*
+ * Frees a shared string.
+ */
+
+void
+string_shared_free_key (struct t_hashtable *hashtable,
+                        void *key, const void *value)
+{
+    /* make C compiler happy */
+    (void) hashtable;
+    (void) value;
+
+    free (key);
+}
+
+/*
+ * Gets a pointer to a shared string.
+ *
+ * A shared string is an entry in the hashtable "string_hashtable_shared", with:
+ * - key: reference count (unsigned integer on 32 bits) + string
+ * - value: NULL pointer (not used)
+ *
+ * The initial reference count is set to 1 and is incremented each time this
+ * function is called for a same string (string content, not the pointer).
+ *
+ * Returns the pointer to the shared string (start of string in key, after the
+ * reference count), NULL if error.
+ * The string returned has exactly same content as string received in argument,
+ * but the pointer to the string is different.
+ *
+ * IMPORTANT: the returned string must NEVER be changed in any way, because it
+ * is used itself as the key of the hashtable.
+ */
+
+const char *
+string_shared_get (const char *string)
+{
+    struct t_hashtable_item *ptr_item;
+    char *key;
+    int length;
+
+    if (!string_hashtable_shared)
+    {
+        /*
+         * use large htable inside hashtable to prevent too many collisions,
+         * which would slow down search of a string in the hashtable
+         */
+        string_hashtable_shared = hashtable_new (1024,
+                                                 WEECHAT_HASHTABLE_POINTER,
+                                                 WEECHAT_HASHTABLE_POINTER,
+                                                 &string_shared_hash_key,
+                                                 &string_shared_keycmp);
+        if (!string_hashtable_shared)
+            return NULL;
+
+        string_hashtable_shared->callback_free_key = &string_shared_free_key;
+    }
+
+    length = sizeof (string_shared_count_t) + strlen (string) + 1;
+    key = malloc (length);
+    if (!key)
+        return NULL;
+    *((string_shared_count_t *)key) = 1;
+    strcpy (key + sizeof (string_shared_count_t), string);
+
+    ptr_item = hashtable_get_item (string_hashtable_shared, key, NULL);
+    if (ptr_item)
+    {
+        /*
+         * the string already exists in the hashtable, then just increase the
+         * reference count on the string
+         */
+        (*((string_shared_count_t *)(ptr_item->key)))++;
+        free (key);
+    }
+    else
+    {
+        /* add the shared string in the hashtable */
+        ptr_item = hashtable_set (string_hashtable_shared, key, NULL);
+        if (!ptr_item)
+            free (key);
+    }
+
+    return (ptr_item) ?
+        ((const char *)ptr_item->key) + sizeof (string_shared_count_t) : NULL;
+}
+
+/*
+ * Frees a shared string.
+ *
+ * The reference count of the string is decremented. If it becomes 0, then the
+ * shared string is removed from the hashtable (and then the string is really
+ * destroyed).
+ */
+
+void
+string_shared_free (const char *string)
+{
+    string_shared_count_t *ptr_count;
+
+    ptr_count = (string_shared_count_t *)(string - sizeof (string_shared_count_t));
+
+    (*ptr_count)--;
+
+    if (*ptr_count == 0)
+        hashtable_remove (string_hashtable_shared, ptr_count);
+}
+
+/*
+ * Frees all allocated data.
+ */
+
+void
+string_end ()
+{
+    if (string_hashtable_shared)
+    {
+        hashtable_free (string_hashtable_shared);
+        string_hashtable_shared = NULL;
+    }
+}
