@@ -30,8 +30,10 @@
 
 #include "weechat.h"
 #include "wee-url.h"
+#include "wee-config.h"
 #include "wee-hashtable.h"
 #include "wee-infolist.h"
+#include "wee-proxy.h"
 #include "wee-string.h"
 
 
@@ -290,6 +292,15 @@ struct t_url_constant url_ssl_version[] =
     { NULL, 0 },
 };
 
+struct t_url_constant url_ssl_options[] =
+{
+#if LIBCURL_VERSION_NUM >= 0x071900
+    /* libcurl >= 7.25.0 */
+    URL_DEF_CONST(SSLOPT, ALLOW_BEAST),
+#endif
+    { NULL, 0 },
+};
+
 struct t_url_constant url_gssapi_delegation[] =
 {
 #if LIBCURL_VERSION_NUM >= 0x071600
@@ -312,6 +323,10 @@ struct t_url_constant url_ssh_auth[] =
     URL_DEF_CONST(SSH_AUTH, KEYBOARD),
     URL_DEF_CONST(SSH_AUTH, DEFAULT),
     URL_DEF_CONST(SSH_AUTH, ANY),
+#endif
+#if LIBCURL_VERSION_NUM >= 0x071C00
+    /* libcurl >= 7.28.0 */
+    URL_DEF_CONST(SSH_AUTH, AGENT),
 #endif
     { NULL, 0 },
 };
@@ -335,6 +350,7 @@ struct t_url_option url_options[] =
     /* libcurl >= 7.21 */
     URL_DEF_OPTION(WILDCARDMATCH, LONG, NULL),
 #endif
+
     /*
      * error options
      */
@@ -342,6 +358,7 @@ struct t_url_option url_options[] =
     /* libcurl >= 7.1 */
     URL_DEF_OPTION(FAILONERROR, LONG, NULL),
 #endif
+
     /*
      * network options
      */
@@ -401,6 +418,13 @@ struct t_url_option url_options[] =
     /* libcurl >= 7.19.0 */
     URL_DEF_OPTION(ADDRESS_SCOPE, LONG, NULL),
 #endif
+#if LIBCURL_VERSION_NUM >= 0x071900
+    /* libcurl >= 7.25.0 */
+    URL_DEF_OPTION(TCP_KEEPALIVE, LONG, NULL),
+    URL_DEF_OPTION(TCP_KEEPIDLE, LONG, NULL),
+    URL_DEF_OPTION(TCP_KEEPINTVL, LONG, NULL),
+#endif
+
     /*
      * name and password options (authentication)
      */
@@ -438,6 +462,7 @@ struct t_url_option url_options[] =
     /* libcurl >= 7.10.7 */
     URL_DEF_OPTION(PROXYAUTH, MASK, url_auth),
 #endif
+
     /*
      * HTTP options
      */
@@ -532,6 +557,7 @@ struct t_url_option url_options[] =
     URL_DEF_OPTION(HTTP_CONTENT_DECODING, LONG, NULL),
     URL_DEF_OPTION(HTTP_TRANSFER_DECODING, LONG, NULL),
 #endif
+
     /*
      * SMTP options
      */
@@ -540,6 +566,11 @@ struct t_url_option url_options[] =
     URL_DEF_OPTION(MAIL_FROM, STRING, NULL),
     /*URL_DEF_OPTION(MAIL_RCPT, LIST, NULL),*/
 #endif
+#if LIBCURL_VERSION_NUM >= 0x071900
+    /* libcurl >= 7.25.0 */
+    URL_DEF_OPTION(MAIL_AUTH, STRING, NULL),
+#endif
+
     /*
      * TFTP options
      */
@@ -547,6 +578,7 @@ struct t_url_option url_options[] =
     /* libcurl >= 7.19.4 */
     URL_DEF_OPTION(TFTP_BLKSIZE, LONG, NULL),
 #endif
+
     /*
      * FTP options
      */
@@ -609,6 +641,7 @@ struct t_url_option url_options[] =
     /* libcurl >= 7.15.1 */
     URL_DEF_OPTION(FTP_FILEMETHOD, LONG, url_ftp_file_method),
 #endif
+
     /*
      * RTSP options
      */
@@ -621,6 +654,7 @@ struct t_url_option url_options[] =
     URL_DEF_OPTION(RTSP_CLIENT_CSEQ, LONG, NULL),
     URL_DEF_OPTION(RTSP_SERVER_CSEQ, LONG, NULL),
 #endif
+
     /*
      * protocol options
      */
@@ -676,6 +710,7 @@ struct t_url_option url_options[] =
     URL_DEF_OPTION(TIMECONDITION, LONG, url_time_condition),
     URL_DEF_OPTION(TIMEVALUE, LONG, NULL),
 #endif
+
     /*
      * connection options
      */
@@ -732,6 +767,7 @@ struct t_url_option url_options[] =
     URL_DEF_OPTION(DNS_SERVERS, STRING, NULL),
     URL_DEF_OPTION(ACCEPTTIMEOUT_MS, LONG, NULL),
 #endif
+
     /*
      * SSL and security options
      */
@@ -796,6 +832,10 @@ struct t_url_option url_options[] =
     /* libcurl >= 7.16.0 */
     URL_DEF_OPTION(SSL_SESSIONID_CACHE, LONG, NULL),
 #endif
+#if LIBCURL_VERSION_NUM >= 0x071900
+    /* libcurl >= 7.25.0 */
+    URL_DEF_OPTION(SSL_OPTIONS, LONG, url_ssl_options),
+#endif
 #if LIBCURL_VERSION_NUM >= 0x071004
     /* libcurl >= 7.16.4 */
     URL_DEF_OPTION(KRBLEVEL, STRING, NULL),
@@ -804,6 +844,7 @@ struct t_url_option url_options[] =
     /* libcurl >= 7.22.0 */
     URL_DEF_OPTION(GSSAPI_DELEGATION, LONG, url_gssapi_delegation),
 #endif
+
     /*
      * SSH options
      */
@@ -824,6 +865,7 @@ struct t_url_option url_options[] =
     /* libcurl >= 7.19.6 */
     URL_DEF_OPTION(SSH_KNOWNHOSTS, STRING, NULL),
 #endif
+
     /*
      * other options
      */
@@ -832,6 +874,7 @@ struct t_url_option url_options[] =
     URL_DEF_OPTION(NEW_FILE_PERMS, LONG, NULL),
     URL_DEF_OPTION(NEW_DIRECTORY_PERMS, LONG, NULL),
 #endif
+
     /*
      * telnet options
      */
@@ -1033,6 +1076,68 @@ weeurl_option_map_cb (void *data,
     return;
 }
 
+/*
+ * Sets proxy in CURL easy handle.
+ */
+
+void
+weeurl_set_proxy (CURL *curl, struct t_proxy *proxy)
+{
+    if (!proxy)
+        return;
+
+    /* set proxy type */
+    switch (CONFIG_INTEGER(proxy->options[PROXY_OPTION_TYPE]))
+    {
+        case PROXY_TYPE_HTTP:
+            curl_easy_setopt (curl, CURLOPT_PROXYTYPE, CURLPROXY_HTTP);
+            break;
+        case PROXY_TYPE_SOCKS4:
+#if LIBCURL_VERSION_NUM >= 0x070A00
+            /* libcurl >= 7.10 */
+            curl_easy_setopt (curl, CURLOPT_PROXYTYPE, CURLPROXY_SOCKS4);
+#else
+            /* proxy socks4 not supported in Curl < 7.10 */
+            return;
+#endif
+            break;
+        case PROXY_TYPE_SOCKS5:
+#if LIBCURL_VERSION_NUM >= 0x070A00
+            /* libcurl >= 7.10 */
+            curl_easy_setopt (curl, CURLOPT_PROXYTYPE, CURLPROXY_SOCKS5);
+#else
+            /* proxy socks4 not supported in Curl < 7.10 */
+            return;
+#endif
+            break;
+    }
+
+    /* set proxy address */
+    curl_easy_setopt (curl, CURLOPT_PROXY,
+                      CONFIG_STRING(proxy->options[PROXY_OPTION_ADDRESS]));
+
+    /* set proxy port */
+    curl_easy_setopt (curl, CURLOPT_PROXYPORT,
+                      CONFIG_INTEGER(proxy->options[PROXY_OPTION_PORT]));
+
+    /* set username/password */
+#if LIBCURL_VERSION_NUM >= 0x071301
+    /* libcurl >= 7.19.1 */
+    if (CONFIG_STRING(proxy->options[PROXY_OPTION_USERNAME])
+        && CONFIG_STRING(proxy->options[PROXY_OPTION_USERNAME])[0])
+    {
+        curl_easy_setopt (curl, CURLOPT_PROXYUSERNAME,
+                          CONFIG_STRING(proxy->options[PROXY_OPTION_USERNAME]));
+    }
+    if (CONFIG_STRING(proxy->options[PROXY_OPTION_PASSWORD])
+        && CONFIG_STRING(proxy->options[PROXY_OPTION_PASSWORD])[0])
+    {
+        curl_easy_setopt (curl, CURLOPT_PROXYPASSWORD,
+                          CONFIG_STRING(proxy->options[PROXY_OPTION_PASSWORD]));
+    }
+#endif
+}
+
 /*
  * Downloads URL using options.
  *
@@ -1054,6 +1159,7 @@ weeurl_download (const char *url, struct t_hashtable *options)
     CURLoption url_file_opt_func[2] = { CURLOPT_READFUNCTION, CURLOPT_WRITEFUNCTION };
     CURLoption url_file_opt_data[2] = { CURLOPT_READDATA, CURLOPT_WRITEDATA };
     void *url_file_opt_cb[2] = { &weeurl_read, &weeurl_write };
+    struct t_proxy *ptr_proxy;
     int rc, i;
 
     rc = 0;
@@ -1081,6 +1187,15 @@ weeurl_download (const char *url, struct t_hashtable *options)
     curl_easy_setopt (curl, CURLOPT_URL, url);
     curl_easy_setopt (curl, CURLOPT_FOLLOWLOCATION, 1L);
 
+    /* set proxy (if option weechat.network.proxy_curl is set) */
+    if (CONFIG_STRING(config_network_proxy_curl)
+        && CONFIG_STRING(config_network_proxy_curl)[0])
+    {
+        ptr_proxy = proxy_search (CONFIG_STRING(config_network_proxy_curl));
+        if (ptr_proxy)
+            weeurl_set_proxy (curl, ptr_proxy);
+    }
+
     /* set file in/out from options in hashtable */
     if (options)
     {
