@@ -197,9 +197,9 @@ gui_mouse_grab_end (const char *mouse_key)
         }
         gui_input_insert_string (gui_current_window->buffer,
                                  mouse_key_input, -1);
-        if (gui_current_window->buffer->completion)
-            gui_completion_stop (gui_current_window->buffer->completion, 1);
-        gui_input_text_changed_modifier_and_signal (gui_current_window->buffer, 1);
+        gui_input_text_changed_modifier_and_signal (gui_current_window->buffer,
+                                                    1, /* save undo */
+                                                    1); /* stop completion */
     }
 
     gui_mouse_grab = 0;
