@@ -32,6 +32,7 @@
 
 #include "../core/weechat.h"
 #include "../core/wee-config.h"
+#include "../core/wee-eval.h"
 #include "../core/wee-hashtable.h"
 #include "../core/wee-hook.h"
 #include "../core/wee-string.h"
@@ -65,12 +66,19 @@ char *gui_chat_lines_waiting_buffer = NULL;     /* lines waiting for core   */
 void
 gui_chat_init ()
 {
-    /* build empty prefixes */
-    gui_chat_prefix[GUI_CHAT_PREFIX_ERROR] = strdup (gui_chat_prefix_empty);
-    gui_chat_prefix[GUI_CHAT_PREFIX_NETWORK] = strdup (gui_chat_prefix_empty);
-    gui_chat_prefix[GUI_CHAT_PREFIX_ACTION] = strdup (gui_chat_prefix_empty);
-    gui_chat_prefix[GUI_CHAT_PREFIX_JOIN] = strdup (gui_chat_prefix_empty);
-    gui_chat_prefix[GUI_CHAT_PREFIX_QUIT] = strdup (gui_chat_prefix_empty);
+    char *default_prefix[GUI_CHAT_NUM_PREFIXES] =
+        { GUI_CHAT_PREFIX_ERROR_DEFAULT, GUI_CHAT_PREFIX_NETWORK_DEFAULT,
+          GUI_CHAT_PREFIX_ACTION_DEFAULT, GUI_CHAT_PREFIX_JOIN_DEFAULT,
+          GUI_CHAT_PREFIX_QUIT_DEFAULT };
+    char str_prefix[64];
+    int i;
+
+    /* build default prefixes */
+    for (i = 0; i < GUI_CHAT_NUM_PREFIXES; i++)
+    {
+        snprintf (str_prefix, sizeof (str_prefix), "%s\t", default_prefix[i]);
+        gui_chat_prefix[i] = strdup (str_prefix);
+    }
 
     /* some hsignals */
     hook_hsignal (NULL, "chat_quote_time_prefix_message",
@@ -108,11 +116,11 @@ gui_chat_prefix_build ()
         pos_color = strstr (ptr_prefix, "${");
 
         snprintf(prefix, sizeof (prefix), "%s%s\t",
-                 (!pos_color || (pos_color > ptr_prefix)) ? GUI_COLOR(prefix_color[i]) : "",
+                 (ptr_prefix[0] && (!pos_color || (pos_color > ptr_prefix))) ? GUI_COLOR(prefix_color[i]) : "",
                  ptr_prefix);
 
         if (pos_color)
-            gui_chat_prefix[i] = gui_color_string_replace_colors (prefix);
+            gui_chat_prefix[i] = eval_expression (prefix, NULL, NULL, NULL);
         else
             gui_chat_prefix[i] = strdup (prefix);
     }
@@ -233,6 +241,8 @@ gui_chat_string_add_offset_screen (const char *string, int offset_screen)
 /*
  * Gets real position in string (ignoring formatting chars like
  * colors/attributes).
+ *
+ * Returns real position, in bytes.
  */
 
 int
@@ -267,6 +277,36 @@ gui_chat_string_real_pos (const char *string, int pos)
     return 0 + (real_pos - string);
 }
 
+/*
+ * Gets real position in string (ignoring formatting chars like
+ * colors/attributes).
+ *
+ * Returns position, in number of UTF-8 chars.
+ */
+
+int
+gui_chat_string_pos (const char *string, int real_pos)
+{
+    const char *ptr_string, *limit;
+    int count;
+
+    count = 0;
+    ptr_string = string;
+    limit = ptr_string + real_pos;
+    while (ptr_string && ptr_string[0] && (ptr_string < limit))
+    {
+        ptr_string = gui_chat_string_next_char (NULL, NULL,
+                                                (unsigned char *)ptr_string,
+                                                0, 0, 0);
+        if (ptr_string)
+        {
+            ptr_string = utf8_next_char (ptr_string);
+            count++;
+        }
+    }
+    return count;
+}
+
 /*
  * Returns info about next word: beginning, end, length.
  *
@@ -366,7 +406,7 @@ gui_chat_get_time_string (time_t date)
 
     if (strstr (text_time, "${"))
     {
-        text_with_color = gui_color_string_replace_colors (text_time);
+        text_with_color = eval_expression (text_time, NULL, NULL, NULL);
         if (text_with_color)
         {
             if (strcmp (text_time, text_with_color) != 0)
@@ -884,7 +924,7 @@ gui_chat_print_lines_waiting_buffer ()
         {
             for (i = 0; i < num_lines; i++)
             {
-                gui_chat_printf (NULL, lines[i]);
+                gui_chat_printf (NULL, "%s", lines[i]);
             }
             string_free_split (lines);
         }
