@@ -30,6 +30,12 @@ struct t_gui_line;
 
 #define GUI_CHAT_TAG_NO_HIGHLIGHT "no_highlight"
 
+#define GUI_CHAT_PREFIX_ERROR_DEFAULT   "=!="
+#define GUI_CHAT_PREFIX_NETWORK_DEFAULT "--"
+#define GUI_CHAT_PREFIX_ACTION_DEFAULT  " *"
+#define GUI_CHAT_PREFIX_JOIN_DEFAULT    "-->"
+#define GUI_CHAT_PREFIX_QUIT_DEFAULT    "<--"
+
 enum t_gui_chat_prefix
 {
     GUI_CHAT_PREFIX_ERROR = 0,
@@ -65,6 +71,7 @@ extern char *gui_chat_string_add_offset (const char *string, int offset);
 extern char *gui_chat_string_add_offset_screen (const char *string,
                                                 int offset_screen);
 extern int gui_chat_string_real_pos (const char *string, int pos);
+extern int gui_chat_string_pos (const char *string, int real_pos);
 extern void gui_chat_get_word_info (struct t_gui_window *window,
                                     const char *data, int *word_start_offset,
                                     int *word_end_offset,
