@@ -32,7 +32,8 @@ extern void gui_input_replace_input (struct t_gui_buffer *buffer,
                                      const char *new_input);
 extern void gui_input_paste_pending_signal ();
 extern void gui_input_text_changed_modifier_and_signal (struct t_gui_buffer *buffer,
-                                                        int save_undo);
+                                                        int save_undo,
+                                                        int stop_completion);
 extern void gui_input_set_pos (struct t_gui_buffer *buffer, int pos);
 extern int gui_input_insert_string (struct t_gui_buffer *buffer,
                                     const char *string, int pos);
@@ -43,9 +44,12 @@ extern void gui_input_return (struct t_gui_buffer *buffer);
 extern void gui_input_complete_next (struct t_gui_buffer *buffer);
 extern void gui_input_complete_previous (struct t_gui_buffer *buffer);
 extern void gui_input_search_text (struct t_gui_buffer *buffer);
+extern void gui_input_search_compile_regex (struct t_gui_buffer *buffer);
+extern void gui_input_search_switch_case (struct t_gui_buffer *buffer);
+extern void gui_input_search_switch_regex (struct t_gui_buffer *buffer);
+extern void gui_input_search_switch_where (struct t_gui_buffer *buffer);
 extern void gui_input_search_previous (struct t_gui_buffer *buffer);
 extern void gui_input_search_next (struct t_gui_buffer *buffer);
-extern void gui_input_search_switch_case (struct t_gui_buffer *buffer);
 extern void gui_input_search_stop (struct t_gui_buffer *buffer);
 extern void gui_input_delete_previous_char (struct t_gui_buffer *buffer);
 extern void gui_input_delete_next_char (struct t_gui_buffer *buffer);
