@@ -580,6 +580,39 @@ gui_key_set_score (struct t_gui_key *key)
     key->score = score;
 }
 
+/*
+ * Checks if a key is safe or not: a safe key begins always with the "meta" or
+ * "ctrl" code (except "@" allowed in cursor/mouse contexts).
+ *
+ * Returns:
+ *   1: key is safe
+ *   0: key is NOT safe
+ */
+
+int
+gui_key_is_safe (int context, const char *key)
+{
+    char *internal_code;
+    int rc;
+
+    /* "@" is allowed at beginning for cursor/mouse contexts */
+    if ((key[0] == '@')
+        && ((context == GUI_KEY_CONTEXT_CURSOR)
+            || (context == GUI_KEY_CONTEXT_MOUSE)))
+    {
+        return 1;
+    }
+
+    /* check that first char is a ctrl or meta code */
+    internal_code = gui_key_get_internal_code (key);
+    if (!internal_code)
+        return 0;
+    rc = (internal_code[0] == '\x01') ? 1 : 0;
+    free (internal_code);
+
+    return rc;
+}
+
 /*
  * Adds a new key in keys list.
  *
@@ -731,7 +764,7 @@ gui_key_search_part (struct t_gui_buffer *buffer, int context,
 }
 
 /*
- * Binds a key to a function (command or special function).
+ * Binds a key to a command.
  *
  * If buffer is not null, then key is specific to buffer otherwise it's general
  * key (for most keys).
@@ -1007,11 +1040,14 @@ gui_key_focus_command (const char *key, int context,
                        struct t_hashtable **hashtable_focus)
 {
     struct t_gui_key *ptr_key;
-    int i, errors, matching, debug;
+    int i, errors, matching, debug, rc;
+    long unsigned int value;
     char *command, **commands;
+    const char *str_buffer;
     struct t_hashtable *hashtable;
     struct t_weelist *list_keys;
     struct t_weelist_item *ptr_item;
+    struct t_gui_buffer *ptr_buffer;
 
     debug = 0;
     if (gui_cursor_debug && (context == GUI_KEY_CONTEXT_CURSOR))
@@ -1049,6 +1085,18 @@ gui_key_focus_command (const char *key, int context,
         if (!hashtable)
             continue;
 
+        /* get buffer */
+        ptr_buffer = gui_current_window->buffer;
+        str_buffer = hashtable_get (hashtable, "_buffer");
+        if (str_buffer && str_buffer[0])
+        {
+            rc = sscanf (str_buffer, "%lx", &value);
+            if ((rc != EOF) && (rc != 0))
+                ptr_buffer = (struct t_gui_buffer *)value;
+        }
+        if (!ptr_buffer)
+            continue;
+
         if ((context == GUI_KEY_CONTEXT_CURSOR) && gui_cursor_debug)
         {
             gui_input_delete_line (gui_current_window->buffer);
@@ -1092,6 +1140,7 @@ gui_key_focus_command (const char *key, int context,
                 else
                 {
                     command = string_replace_with_callback (commands[i],
+                                                            "${", "}",
                                                             &gui_key_focus_command_replace_cb,
                                                             hashtable,
                                                             &errors);
@@ -1102,10 +1151,12 @@ gui_key_focus_command (const char *key, int context,
                             if (debug)
                             {
                                 gui_chat_printf (NULL,
-                                                 _("Executing command: \"%s\""),
-                                                 command);
+                                                 _("Executing command: \"%s\" "
+                                                   "on buffer \"%s\""),
+                                                 command,
+                                                 ptr_buffer->full_name);
                             }
-                            input_data (gui_current_window->buffer, command);
+                            input_data (ptr_buffer, command);
                         }
                         free (command);
                     }
@@ -1288,7 +1339,7 @@ gui_key_pressed (const char *key_str)
     {
         if (strcmp (ptr_key->key, gui_key_combo_buffer) == 0)
         {
-            /* exact combo found => execute function or command */
+            /* exact combo found => execute command */
             gui_key_combo_buffer[0] = '\0';
             if (ptr_key->command)
             {
