@@ -22,6 +22,9 @@
 
 /* main functions (GUI dependent) */
 
+extern void gui_main_get_password (const char *prompt1, const char *prompt2,
+                                   const char *prompt3,
+                                   char *password, int size);
 extern void gui_main_loop ();
 extern void gui_main_pre_init (int *argc, char **argv[]);
 extern void gui_main_init ();
