@@ -1010,9 +1010,10 @@ weechat_plugin_init (struct t_weechat_plugin *plugin, int argc, char *argv[])
                              "      $n-m: arguments from 'n' to 'm'\n"
                              "        $*: all arguments\n"
                              "        $~: last argument\n"
-                             "     $nick: current nick\n"
-                             "  $channel: current channel\n"
-                             "   $server: current server\n\n"
+                             "      $var: where \"var\" is a local variable of "
+                             "buffer (see /buffer localvar)\n"
+                             "            examples: $nick, $channel, $server, "
+                             "$plugin, $name\n\n"
                              "To remove an alias, use command /unalias.\n\n"
                              "Examples:\n"
                              "  alias /split to split window horizontally:\n"
