@@ -53,13 +53,14 @@ char *guile_stdout = NULL;
 
 struct t_guile_function
 {
-    SCM proc;
-    SCM args;
+    SCM proc;                          /* proc to call                      */
+    SCM *argv;                         /* arguments for proc                */
+    size_t nargs;                      /* length of arguments               */
 };
 
 /*
  * string used to execute action "install":
- * when signal "guile_install_script" is received, name of string
+ * when signal "guile_script_install" is received, name of string
  * is added to this string, to be installed later by a timer (when nothing is
  * running in script)
  */
@@ -67,12 +68,20 @@ char *guile_action_install_list = NULL;
 
 /*
  * string used to execute action "remove":
- * when signal "guile_remove_script" is received, name of string
+ * when signal "guile_script_remove" is received, name of string
  * is added to this string, to be removed later by a timer (when nothing is
  * running in script)
  */
 char *guile_action_remove_list = NULL;
 
+/*
+ * string used to execute action "autoload":
+ * when signal "guile_script_autoload" is received, name of string
+ * is added to this string, to autoload or disable autoload later by a timer
+ * (when nothing is running in script)
+ */
+char *guile_action_autoload_list = NULL;
+
 
 /*
  * Flushes stdout.
@@ -109,17 +118,18 @@ weechat_guile_catch (void *procedure, void *data)
 }
 
 /*
- * Encapsulates call to scm_call_1 (to give arguments).
+ * Encapsulates call to scm_call_n (to give arguments).
  */
 
 SCM
-weechat_guile_scm_call_1 (void *proc)
+weechat_guile_scm_call_n (void *proc)
 {
     struct t_guile_function *guile_function;
 
     guile_function = (struct t_guile_function *)proc;
 
-    return scm_call_1 (guile_function->proc, guile_function->args);
+    return scm_call_n (guile_function->proc,
+                       guile_function->argv, guile_function->nargs);
 }
 
 /*
@@ -127,7 +137,7 @@ weechat_guile_scm_call_1 (void *proc)
  */
 
 SCM
-weechat_guile_exec_function (const char *function, SCM args)
+weechat_guile_exec_function (const char *function, SCM *argv, size_t nargs)
 {
     SCM func, func2, value;
     struct t_guile_function guile_function;
@@ -135,11 +145,12 @@ weechat_guile_exec_function (const char *function, SCM args)
     func = weechat_guile_catch (scm_c_lookup, (void *)function);
     func2 = weechat_guile_catch (scm_variable_ref, func);
 
-    if (args)
+    if (argv)
     {
         guile_function.proc = func2;
-        guile_function.args = args;
-        value = weechat_guile_catch (weechat_guile_scm_call_1, &guile_function);
+        guile_function.argv = argv;
+        guile_function.nargs = nargs;
+        value = weechat_guile_catch (weechat_guile_scm_call_n, &guile_function);
     }
     else
     {
@@ -255,12 +266,17 @@ weechat_guile_exec (struct t_plugin_script *script,
                     char *format, void **argv)
 {
     struct t_plugin_script *old_guile_current_script;
-    SCM argv_list, rc;
+    SCM rc, old_current_module;
     void *argv2[17], *ret_value;
     int i, argc, *ret_int;
 
     old_guile_current_script = guile_current_script;
-    scm_set_current_module ((SCM)(script->interpreter));
+    old_current_module = NULL;
+    if (script->interpreter)
+    {
+        old_current_module = scm_current_module ();
+        scm_set_current_module ((SCM)(script->interpreter));
+    }
     guile_current_script = script;
 
     if (argv && argv[0])
@@ -285,20 +301,11 @@ weechat_guile_exec (struct t_plugin_script *script,
         {
             argv2[i] = SCM_UNDEFINED;
         }
-        argv_list = scm_list_n (argv2[0], argv2[1],
-                                argv2[2], argv2[3],
-                                argv2[4], argv2[5],
-                                argv2[6], argv2[7],
-                                argv2[8], argv2[9],
-                                argv2[10], argv2[11],
-                                argv2[12], argv2[13],
-                                argv2[14], argv2[15],
-                                argv2[16]);
-        rc = weechat_guile_exec_function (function, argv_list);
+        rc = weechat_guile_exec_function (function, (SCM *)argv2, argc);
     }
     else
     {
-        rc = weechat_guile_exec_function (function, NULL);
+        rc = weechat_guile_exec_function (function, NULL, 0);
     }
 
     ret_value = NULL;
@@ -336,8 +343,8 @@ weechat_guile_exec (struct t_plugin_script *script,
                         weechat_prefix ("error"), GUILE_PLUGIN_NAME, function);
     }
 
-    if (old_guile_current_script)
-        scm_set_current_module ((SCM)(old_guile_current_script->interpreter));
+    if (old_current_module)
+        scm_set_current_module (old_current_module);
 
     guile_current_script = old_guile_current_script;
 
@@ -816,6 +823,12 @@ weechat_guile_timer_action_cb (void *data, int remaining_calls)
                                          &guile_quiet,
                                          &guile_action_remove_list);
         }
+        else if (data == &guile_action_autoload_list)
+        {
+            plugin_script_action_autoload (weechat_guile_plugin,
+                                           &guile_quiet,
+                                           &guile_action_autoload_list);
+        }
     }
 
     return WEECHAT_RC_OK;
@@ -851,6 +864,14 @@ weechat_guile_signal_script_action_cb (void *data, const char *signal,
                                 &weechat_guile_timer_action_cb,
                                 &guile_action_remove_list);
         }
+        else if (strcmp (signal, "guile_script_autoload") == 0)
+        {
+            plugin_script_action_add (&guile_action_autoload_list,
+                                      (const char *)signal_data);
+            weechat_hook_timer (1, 0, 1,
+                                &weechat_guile_timer_action_cb,
+                                &guile_action_autoload_list);
+        }
     }
 
     return WEECHAT_RC_OK;
@@ -948,7 +969,7 @@ weechat_plugin_init (struct t_weechat_plugin *plugin, int argc, char *argv[])
     plugin_script_display_short_list (weechat_guile_plugin,
                                       guile_scripts);
 
-    /* init ok */
+    /* init OK */
     return WEECHAT_RC_OK;
 }
 
@@ -972,6 +993,8 @@ weechat_plugin_end (struct t_weechat_plugin *plugin)
         free (guile_action_install_list);
     if (guile_action_remove_list)
         free (guile_action_remove_list);
+    if (guile_action_autoload_list)
+        free (guile_action_autoload_list);
 
     return WEECHAT_RC_OK;
 }
