@@ -42,12 +42,14 @@ struct t_irc_channel_speaking
 struct t_irc_channel
 {
     int type;                          /* channel type                      */
-    char *name;                        /* name of channel (exemple: "#abc") */
+    char *name;                        /* name of channel (example: "#abc") */
     char *topic;                       /* topic of channel (host for pv)    */
     char *modes;                       /* channel modes                     */
     int limit;                         /* user limit (0 is limit not set)   */
     char *key;                         /* channel key (NULL if no key set)  */
-    int names_received;                /* names received (message 366)      */
+    struct t_hashtable *join_msg_received; /* messages received after join: */
+                                       /* 366=names, 332/333=topic,         */
+                                       /* 329=creation date                 */
     int checking_away;                 /* = 1 if checking away with WHO cmd */
     char *away_message;                /* to display away only once in pv   */
     int has_quit_server;               /* =1 if nick has quit (pv only), to */
@@ -67,6 +69,7 @@ struct t_irc_channel
     struct t_irc_channel_speaking *nicks_speaking_time; /* for smart filter */
                                        /* of join/part/quit messages        */
     struct t_irc_channel_speaking *last_nick_speaking_time;
+    struct t_hashtable *join_smart_filtered; /* smart filtered joins        */
     struct t_gui_buffer *buffer;       /* buffer allocated for channel      */
     char *buffer_as_string;            /* used to return buffer info        */
     struct t_irc_channel *prev_channel; /* link to previous channel         */
@@ -105,6 +108,9 @@ extern void irc_channel_nick_speaking_add (struct t_irc_channel *channel,
 extern void irc_channel_nick_speaking_rename (struct t_irc_channel *channel,
                                               const char *old_nick,
                                               const char *new_nick);
+extern void irc_channel_nick_speaking_rename_if_present (struct t_irc_server *server,
+                                                         struct t_irc_channel *channel,
+                                                         const char *nick_name);
 extern struct t_irc_channel_speaking *irc_channel_nick_speaking_time_search (struct t_irc_server *server,
                                                                              struct t_irc_channel *channel,
                                                                              const char *nick_name,
@@ -118,6 +124,16 @@ extern void irc_channel_nick_speaking_time_rename (struct t_irc_server *server,
                                                    struct t_irc_channel *channel,
                                                    const char *old_nick,
                                                    const char *new_nick);
+extern void irc_channel_join_smart_filtered_add (struct t_irc_channel *channel,
+                                                 const char *nick,
+                                                 time_t join_time);
+extern void irc_channel_join_smart_filtered_rename (struct t_irc_channel *channel,
+                                                    const char *old_nick,
+                                                    const char *new_nick);
+extern void irc_channel_join_smart_filtered_remove (struct t_irc_channel *channel,
+                                                    const char *nick);
+extern void irc_channel_join_smart_filtered_unmask (struct t_irc_channel *channel,
+                                                    const char *nick);
 extern void irc_channel_rejoin (struct t_irc_server *server,
                                 struct t_irc_channel *channel);
 extern int irc_channel_autorejoin_cb (void *data, int remaining_calls);
