@@ -62,6 +62,12 @@ enum t_irc_config_look_notice_as_pv
     IRC_CONFIG_LOOK_NOTICE_AS_PV_ALWAYS,
 };
 
+enum t_irc_config_look_nick_color_hash
+{
+    IRC_CONFIG_LOOK_NICK_COLOR_HASH_DJB2 = 0,
+    IRC_CONFIG_LOOK_NICK_COLOR_HASH_SUM,
+};
+
 enum t_irc_config_look_nick_mode
 {
     IRC_CONFIG_LOOK_NICK_MODE_NONE = 0,
@@ -97,15 +103,6 @@ extern struct t_config_option *irc_config_look_color_nicks_in_nicklist;
 extern struct t_config_option *irc_config_look_color_nicks_in_server_messages;
 extern struct t_config_option *irc_config_look_color_pv_nick_like_channel;
 extern struct t_config_option *irc_config_look_ctcp_time_format;
-extern struct t_config_option *irc_config_look_server_buffer;
-extern struct t_config_option *irc_config_look_pv_buffer;
-extern struct t_config_option *irc_config_look_new_channel_position;
-extern struct t_config_option *irc_config_look_new_pv_position;
-extern struct t_config_option *irc_config_look_nick_mode;
-extern struct t_config_option *irc_config_look_nick_mode_empty;
-extern struct t_config_option *irc_config_look_nick_color_force;
-extern struct t_config_option *irc_config_look_nick_color_stop_chars;
-extern struct t_config_option *irc_config_look_nick_completion_smart;
 extern struct t_config_option *irc_config_look_display_away;
 extern struct t_config_option *irc_config_look_display_ctcp_blocked;
 extern struct t_config_option *irc_config_look_display_ctcp_reply;
@@ -117,43 +114,55 @@ extern struct t_config_option *irc_config_look_display_join_message;
 extern struct t_config_option *irc_config_look_display_old_topic;
 extern struct t_config_option *irc_config_look_display_pv_away_once;
 extern struct t_config_option *irc_config_look_display_pv_back;
-extern struct t_config_option *irc_config_look_item_away_message;
-extern struct t_config_option *irc_config_look_item_channel_modes_hide_key;
-extern struct t_config_option *irc_config_look_item_nick_modes;
-extern struct t_config_option *irc_config_look_item_nick_prefix;
 extern struct t_config_option *irc_config_look_highlight_server;
 extern struct t_config_option *irc_config_look_highlight_channel;
 extern struct t_config_option *irc_config_look_highlight_pv;
 extern struct t_config_option *irc_config_look_highlight_tags;
+extern struct t_config_option *irc_config_look_item_away_message;
+extern struct t_config_option *irc_config_look_item_channel_modes_hide_key;
 extern struct t_config_option *irc_config_look_item_display_server;
+extern struct t_config_option *irc_config_look_item_nick_modes;
+extern struct t_config_option *irc_config_look_item_nick_prefix;
 extern struct t_config_option *irc_config_look_msgbuffer_fallback;
+extern struct t_config_option *irc_config_look_new_channel_position;
+extern struct t_config_option *irc_config_look_new_pv_position;
 extern struct t_config_option *irc_config_look_nicks_hide_password;
+extern struct t_config_option *irc_config_look_nick_color_force;
+extern struct t_config_option *irc_config_look_nick_color_hash;
+extern struct t_config_option *irc_config_look_nick_color_stop_chars;
+extern struct t_config_option *irc_config_look_nick_completion_smart;
+extern struct t_config_option *irc_config_look_nick_mode;
+extern struct t_config_option *irc_config_look_nick_mode_empty;
 extern struct t_config_option *irc_config_look_notice_as_pv;
+extern struct t_config_option *irc_config_look_notice_welcome_redirect;
 extern struct t_config_option *irc_config_look_notify_tags_ison;
 extern struct t_config_option *irc_config_look_notify_tags_whois;
 extern struct t_config_option *irc_config_look_part_closes_buffer;
+extern struct t_config_option *irc_config_look_pv_buffer;
+extern struct t_config_option *irc_config_look_pv_tags;
 extern struct t_config_option *irc_config_look_raw_messages;
+extern struct t_config_option *irc_config_look_server_buffer;
 extern struct t_config_option *irc_config_look_smart_filter;
 extern struct t_config_option *irc_config_look_smart_filter_delay;
 extern struct t_config_option *irc_config_look_smart_filter_join;
 extern struct t_config_option *irc_config_look_smart_filter_join_unmask;
-extern struct t_config_option *irc_config_look_smart_filter_quit;
 extern struct t_config_option *irc_config_look_smart_filter_nick;
+extern struct t_config_option *irc_config_look_smart_filter_quit;
 extern struct t_config_option *irc_config_look_topic_strip_colors;
 
-extern struct t_config_option *irc_config_color_message_join;
-extern struct t_config_option *irc_config_color_message_quit;
-extern struct t_config_option *irc_config_color_mirc_remap;
-extern struct t_config_option *irc_config_color_nick_prefixes;
-extern struct t_config_option *irc_config_color_notice;
 extern struct t_config_option *irc_config_color_input_nick;
 extern struct t_config_option *irc_config_color_item_away;
 extern struct t_config_option *irc_config_color_item_channel_modes;
 extern struct t_config_option *irc_config_color_item_lag_counting;
 extern struct t_config_option *irc_config_color_item_lag_finished;
+extern struct t_config_option *irc_config_color_message_join;
+extern struct t_config_option *irc_config_color_message_quit;
+extern struct t_config_option *irc_config_color_mirc_remap;
+extern struct t_config_option *irc_config_color_nick_prefixes;
+extern struct t_config_option *irc_config_color_notice;
 extern struct t_config_option *irc_config_color_reason_quit;
-extern struct t_config_option *irc_config_color_topic_old;
 extern struct t_config_option *irc_config_color_topic_new;
+extern struct t_config_option *irc_config_color_topic_old;
 
 extern struct t_config_option *irc_config_network_alternate_nick;
 extern struct t_config_option *irc_config_network_autoreconnect_delay_growing;
