@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2012 Sebastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2013 Sebastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -23,6 +23,7 @@
 #define IRC_PROTOCOL_CALLBACK(__command)                                \
     int                                                                 \
     irc_protocol_cb_##__command (struct t_irc_server *server,           \
+                                 time_t date,                           \
                                  const char *nick,                      \
                                  const char *address,                   \
                                  const char *host,                      \
@@ -33,6 +34,7 @@
                                  char **argv_eol)
 
 #define IRC_PROTOCOL_MIN_ARGS(__min_args)                               \
+    (void) date;                                                        \
     (void) nick;                                                        \
     (void) address;                                                     \
     (void) host;                                                        \
@@ -65,8 +67,9 @@
 struct t_irc_server;
 
 typedef int (t_irc_recv_func)(struct t_irc_server *server,
-                              const char *nick, const char *address,
-                              const char *host, const char *command,
+                              time_t date, const char *nick,
+                              const char *address, const char *host,
+                              const char *command,
                               int ignored,
                               int argc, char **argv, char **argv_eol);
 
@@ -82,6 +85,7 @@ extern const char *irc_protocol_tags (const char *command, const char *tags,
                                       const char *nick);
 extern void irc_protocol_recv_command (struct t_irc_server *server,
                                        const char *irc_message,
+                                       const char *msg_tags,
                                        const char *msg_command,
                                        const char *msg_channel);
 
