@@ -284,8 +284,11 @@ irc_upgrade_read_cb (void *data,
                      int object_id,
                      struct t_infolist *infolist)
 {
-    int flags, sock, size, i, index;
-    char *buf, option_name[64];
+    int flags, sock, size, i, index, nicks_count, num_items;
+    long number;
+    time_t join_time;
+    char *buf, option_name[64], **nicks, *nick_join, *pos, *error;
+    char **items;
     const char *buffer_name, *str, *nick;
     struct t_irc_nick *ptr_nick;
     struct t_irc_redirect *ptr_redirect;
@@ -418,6 +421,7 @@ irc_upgrade_read_cb (void *data,
                     irc_upgrade_current_server->lag_last_refresh = weechat_infolist_time (infolist, "lag_last_refresh");
                     irc_upgrade_current_server->last_user_message = weechat_infolist_time (infolist, "last_user_message");
                     irc_upgrade_current_server->last_away_check = weechat_infolist_time (infolist, "last_away_check");
+                    irc_upgrade_current_server->last_data_purge = weechat_infolist_time (infolist, "last_data_purge");
                 }
                 break;
             case IRC_UPGRADE_TYPE_CHANNEL:
@@ -439,7 +443,21 @@ irc_upgrade_read_cb (void *data,
                         str = weechat_infolist_string (infolist, "key");
                         if (str)
                             irc_upgrade_current_channel->key = strdup (str);
-                        irc_upgrade_current_channel->names_received = weechat_infolist_integer (infolist, "names_received");
+                        str = weechat_infolist_string (infolist, "join_msg_received");
+                        if (str)
+                        {
+                            items = weechat_string_split (str, ",", 0, 0,
+                                                          &num_items);
+                            if (items)
+                            {
+                                for (i = 0; i < num_items; i++)
+                                {
+                                    weechat_hashtable_set (irc_upgrade_current_channel->join_msg_received,
+                                                           items[i], "1");
+                                }
+                                weechat_string_free_split (items);
+                            }
+                        }
                         irc_upgrade_current_channel->checking_away = weechat_infolist_integer (infolist, "checking_away");
                         str = weechat_infolist_string (infolist, "away_message");
                         if (str)
@@ -481,6 +499,38 @@ irc_upgrade_read_cb (void *data,
                                                                                        option_name));
                             index++;
                         }
+                        str = weechat_infolist_string (infolist, "join_smart_filtered");
+                        if (str)
+                        {
+                            nicks = weechat_string_split (str, ",", 0, 0,
+                                                          &nicks_count);
+                            if (nicks)
+                            {
+                                for (i = 0; i < nicks_count; i++)
+                                {
+                                    pos = strchr (nicks[i], ':');
+                                    if (pos)
+                                    {
+                                        nick_join = weechat_strndup (nicks[i],
+                                                                     pos - nicks[i]);
+                                        if (nick_join)
+                                        {
+                                            error = NULL;
+                                            number = strtol (pos + 1, &error, 10);
+                                            if (error && !error[0])
+                                            {
+                                                join_time = (time_t)number;
+                                                irc_channel_join_smart_filtered_add (irc_upgrade_current_channel,
+                                                                                     nick_join,
+                                                                                     join_time);
+                                            }
+                                            free (nick_join);
+                                        }
+                                    }
+                                }
+                                weechat_string_free_split (nicks);
+                            }
+                        }
                     }
                 }
                 break;
@@ -490,14 +540,11 @@ irc_upgrade_read_cb (void *data,
                     ptr_nick = irc_nick_new (irc_upgrade_current_server,
                                              irc_upgrade_current_channel,
                                              weechat_infolist_string (infolist, "name"),
+                                             weechat_infolist_string (infolist, "host"),
                                              weechat_infolist_string (infolist, "prefixes"),
                                              weechat_infolist_integer (infolist, "away"));
                     if (ptr_nick)
                     {
-                        str = weechat_infolist_string (infolist, "host");
-                        if (str)
-                            ptr_nick->host = strdup (str);
-
                         /*
                          * "flags" is not any more in this infolist (since
                          * WeeChat 0.3.4), but we read it to keep compatibility
