@@ -145,6 +145,9 @@ extern void plugin_script_action_remove (struct t_weechat_plugin *weechat_plugin
                                          void (*script_unload)(struct t_plugin_script *script),
                                          int *quiet,
                                          char **list);
+extern void plugin_script_action_autoload (struct t_weechat_plugin *weechat_plugin,
+                                           int *quiet,
+                                           char **list);
 extern void plugin_script_display_list (struct t_weechat_plugin *weechat_plugin,
                                         struct t_plugin_script *scripts,
                                         const char *name, int full);
