@@ -257,7 +257,7 @@ relay_command_relay (void *data, struct t_gui_buffer *buffer, int argc,
                                 _("%s%s: missing arguments for \"%s\" "
                                   "command"),
                                 weechat_prefix ("error"), RELAY_PLUGIN_NAME,
-                                "relay add");
+                                "relay del");
             }
             return WEECHAT_RC_OK;
         }
@@ -326,7 +326,9 @@ relay_command_init ()
                              "          ssl: enable SSL\n"
                              "protocol.name: protocol and name to relay:\n"
                              "                 - protocol \"irc\": name is the "
-                             "server to share\n"
+                             "server to share (optional, if not given, the server "
+                             "name must be sent by client in command \"PASS\", "
+                             "with format: \"PASS server:password\")\n"
                              "                 - protocol \"weechat\" (name is "
                              "not used)\n"
                              "         port: port used for relay\n"
@@ -340,6 +342,9 @@ relay_command_init ()
                              "    /relay add irc.freenode 8000\n"
                              "  irc proxy, for server \"freenode\", with SSL:\n"
                              "    /relay add ssl.irc.freenode 8001\n"
+                             "  irc proxy, for all servers (client will choose), "
+                             "with SSL:\n"
+                             "    /relay add ssl.irc 8002\n"
                              "  weechat protocol:\n"
                              "    /relay add weechat 9000\n"
                              "  weechat protocol with SSL:\n"
