@@ -43,10 +43,10 @@ struct t_config_option *relay_config_look_raw_messages;
 /* relay config, color section */
 
 struct t_config_option *relay_config_color_client;
+struct t_config_option *relay_config_color_status[RELAY_NUM_STATUS];
 struct t_config_option *relay_config_color_text;
 struct t_config_option *relay_config_color_text_bg;
 struct t_config_option *relay_config_color_text_selected;
-struct t_config_option *relay_config_color_status[RELAY_NUM_STATUS];
 
 /* relay config, network section */
 
@@ -61,8 +61,8 @@ struct t_config_option *relay_config_network_websocket_allowed_origins;
 
 /* relay config, irc section */
 
-struct t_config_option *relay_config_irc_backlog_max_number;
 struct t_config_option *relay_config_irc_backlog_max_minutes;
+struct t_config_option *relay_config_irc_backlog_max_number;
 struct t_config_option *relay_config_irc_backlog_since_last_disconnect;
 struct t_config_option *relay_config_irc_backlog_tags;
 struct t_config_option *relay_config_irc_backlog_time_format;
@@ -524,24 +524,6 @@ relay_config_init ()
         N_("text color for client description"),
         NULL, 0, 0, "cyan", NULL, 0,
         NULL, NULL, NULL, NULL, NULL, NULL);
-    relay_config_color_text = weechat_config_new_option (
-        relay_config_file, ptr_section,
-        "text", "color",
-        N_("text color in relay buffer"),
-        NULL, 0, 0, "default", NULL, 0,
-        NULL, NULL, &relay_config_refresh_cb, NULL, NULL, NULL);
-    relay_config_color_text_bg = weechat_config_new_option (
-        relay_config_file, ptr_section,
-        "text_bg", "color",
-        N_("background color in relay buffer"),
-        NULL, 0, 0, "default", NULL, 0,
-        NULL, NULL, &relay_config_refresh_cb, NULL, NULL, NULL);
-    relay_config_color_text_selected = weechat_config_new_option (
-        relay_config_file, ptr_section,
-        "text_selected", "color",
-        N_("text color of selected line in relay buffer"),
-        NULL, 0, 0, "white", NULL, 0,
-        NULL, NULL, &relay_config_refresh_cb, NULL, NULL, NULL);
     relay_config_color_status[RELAY_STATUS_CONNECTING] = weechat_config_new_option (
         relay_config_file, ptr_section,
         "status_connecting", "color",
@@ -572,6 +554,24 @@ relay_config_init ()
         N_("text color for \"disconnected\" status"),
         NULL, 0, 0, "lightred", NULL, 0,
         NULL, NULL, &relay_config_refresh_cb, NULL, NULL, NULL);
+    relay_config_color_text = weechat_config_new_option (
+        relay_config_file, ptr_section,
+        "text", "color",
+        N_("text color in relay buffer"),
+        NULL, 0, 0, "default", NULL, 0,
+        NULL, NULL, &relay_config_refresh_cb, NULL, NULL, NULL);
+    relay_config_color_text_bg = weechat_config_new_option (
+        relay_config_file, ptr_section,
+        "text_bg", "color",
+        N_("background color in relay buffer"),
+        NULL, 0, 0, "default", NULL, 0,
+        NULL, NULL, &relay_config_refresh_cb, NULL, NULL, NULL);
+    relay_config_color_text_selected = weechat_config_new_option (
+        relay_config_file, ptr_section,
+        "text_selected", "color",
+        N_("text color of selected line in relay buffer"),
+        NULL, 0, 0, "white", NULL, 0,
+        NULL, NULL, &relay_config_refresh_cb, NULL, NULL, NULL);
 
     /* section network */
     ptr_section = weechat_config_new_section (relay_config_file, "network",
@@ -629,7 +629,8 @@ relay_config_init ()
         relay_config_file, ptr_section,
         "password", "string",
         N_("password required by clients to access this relay (empty value "
-            "means no password required)"),
+            "means no password required) (note: content is evaluated, see "
+           "/help eval)"),
         NULL, 0, 0, "", NULL, 0, NULL, NULL, NULL, NULL, NULL, NULL);
     relay_config_network_ssl_cert_key = weechat_config_new_option (
         relay_config_file, ptr_section,
@@ -659,13 +660,6 @@ relay_config_init ()
         return 0;
     }
 
-    relay_config_irc_backlog_max_number = weechat_config_new_option (
-        relay_config_file, ptr_section,
-        "backlog_max_number", "integer",
-        N_("maximum number of lines in backlog per IRC channel "
-           "(0 = unlimited)"),
-        NULL, 0, INT_MAX, "256", NULL, 0,
-        NULL, NULL, NULL, NULL, NULL, NULL);
     relay_config_irc_backlog_max_minutes = weechat_config_new_option (
         relay_config_file, ptr_section,
         "backlog_max_minutes", "integer",
@@ -674,6 +668,13 @@ relay_config_init ()
            "43200 = one month, 525600 = one year)"),
         NULL, 0, INT_MAX, "1440", NULL, 0,
         NULL, NULL, NULL, NULL, NULL, NULL);
+    relay_config_irc_backlog_max_number = weechat_config_new_option (
+        relay_config_file, ptr_section,
+        "backlog_max_number", "integer",
+        N_("maximum number of lines in backlog per IRC channel "
+           "(0 = unlimited)"),
+        NULL, 0, INT_MAX, "256", NULL, 0,
+        NULL, NULL, NULL, NULL, NULL, NULL);
     relay_config_irc_backlog_since_last_disconnect = weechat_config_new_option (
         relay_config_file, ptr_section,
         "backlog_since_last_disconnect", "boolean",
