@@ -22,6 +22,18 @@
 
 #define RELAY_WEECHAT_PROTOCOL_SYNC_BUFFER   1
 #define RELAY_WEECHAT_PROTOCOL_SYNC_NICKLIST 2
+#define RELAY_WEECHAT_PROTOCOL_SYNC_BUFFERS  4
+#define RELAY_WEECHAT_PROTOCOL_SYNC_UPGRADE  8
+
+#define RELAY_WEECHAT_PROTOCOL_SYNC_ALL         \
+    (RELAY_WEECHAT_PROTOCOL_SYNC_BUFFER |       \
+     RELAY_WEECHAT_PROTOCOL_SYNC_NICKLIST |     \
+     RELAY_WEECHAT_PROTOCOL_SYNC_BUFFERS |      \
+     RELAY_WEECHAT_PROTOCOL_SYNC_UPGRADE)
+
+#define RELAY_WEECHAT_PROTOCOL_SYNC_FOR_BUFFER  \
+    (RELAY_WEECHAT_PROTOCOL_SYNC_BUFFER |       \
+     RELAY_WEECHAT_PROTOCOL_SYNC_NICKLIST)
 
 #define RELAY_WEECHAT_PROTOCOL_CALLBACK(__command)                      \
     int                                                                 \
@@ -77,6 +89,9 @@ extern int relay_weechat_protocol_signal_nicklist_cb (void *data,
                                                       const char *signal,
                                                       const char *type_data,
                                                       void *signal_data);
+extern int relay_weechat_protocol_hsignal_nicklist_cb (void *data,
+                                                       const char *signal,
+                                                       struct t_hashtable *hashtable);
 extern int relay_weechat_protocol_signal_upgrade_cb (void *data,
                                                      const char *signal,
                                                      const char *type_data,
@@ -84,6 +99,6 @@ extern int relay_weechat_protocol_signal_upgrade_cb (void *data,
 extern int relay_weechat_protocol_timer_nicklist_cb (void *data,
                                                      int remaining_calls);
 extern void relay_weechat_protocol_recv (struct t_relay_client *client,
-                                         char *data);
+                                         const char *data);
 
 #endif /* __WEECHAT_RELAY_WEECHAT_PROTOCOL_H */
