@@ -167,9 +167,10 @@ void
 relay_weechat_alloc (struct t_relay_client *client)
 {
     struct t_relay_weechat_data *weechat_data;
-    const char *password;
+    char *password;
 
-    password = weechat_config_string (relay_config_network_password);
+    password = weechat_string_eval_expression (weechat_config_string (relay_config_network_password),
+                                               NULL, NULL, NULL);
 
     client->protocol_data = malloc (sizeof (*weechat_data));
     if (client->protocol_data)
@@ -198,6 +199,9 @@ relay_weechat_alloc (struct t_relay_client *client)
 
         relay_weechat_hook_signals (client);
     }
+
+    if (password)
+        free (password);
 }
 
 /*
