@@ -28,28 +28,29 @@ extern struct t_config_file *xfer_config;
 
 extern struct t_config_option *xfer_config_look_auto_open_buffer;
 extern struct t_config_option *xfer_config_look_progress_bar_size;
+extern struct t_config_option *xfer_config_look_pv_tags;
 
+extern struct t_config_option *xfer_config_color_status[];
 extern struct t_config_option *xfer_config_color_text;
 extern struct t_config_option *xfer_config_color_text_bg;
 extern struct t_config_option *xfer_config_color_text_selected;
-extern struct t_config_option *xfer_config_color_status[];
 
-extern struct t_config_option *xfer_config_network_timeout;
 extern struct t_config_option *xfer_config_network_blocksize;
 extern struct t_config_option *xfer_config_network_fast_send;
-extern struct t_config_option *xfer_config_network_port_range;
 extern struct t_config_option *xfer_config_network_own_ip;
+extern struct t_config_option *xfer_config_network_port_range;
 extern struct t_config_option *xfer_config_network_speed_limit;
+extern struct t_config_option *xfer_config_network_timeout;
 
+struct t_config_option *xfer_config_file_auto_accept_chats;
+struct t_config_option *xfer_config_file_auto_accept_files;
+struct t_config_option *xfer_config_file_auto_accept_nicks;
+extern struct t_config_option *xfer_config_file_auto_rename;
+extern struct t_config_option *xfer_config_file_auto_resume;
+extern struct t_config_option *xfer_config_file_convert_spaces;
 extern struct t_config_option *xfer_config_file_download_path;
 extern struct t_config_option *xfer_config_file_upload_path;
 extern struct t_config_option *xfer_config_file_use_nick_in_filename;
-extern struct t_config_option *xfer_config_file_convert_spaces;
-extern struct t_config_option *xfer_config_file_auto_rename;
-extern struct t_config_option *xfer_config_file_auto_resume;
-extern struct t_config_option *xfer_config_file_auto_accept_files;
-extern struct t_config_option *xfer_config_file_auto_accept_chats;
-extern struct t_config_option *xfer_config_file_auto_accept_nicks;
 
 extern int xfer_config_init ();
 extern int xfer_config_read ();
